# Copyright (C) 2008-2012 Bernd Holleczek
#                         Saarland Cancer Registry
#                         b.holleczek@gbe-ekr.saarland.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See file LICENSE for more details.
#
#
 
hybridperiod <- function(data, k, surv.m, surv.f, diagbeg, perbeg, perend, agedist=NULL, group=NULL) {

  #check and coerce argument data
  LABEL_data <- deparse(substitute(data))
  (missing(data) || is.null(data)) && stop(gettextf("missing argument data."))
  nrow(data)==0 && stop(gettextf("argument data=%s: no observations.", LABEL_data))
  class(data)!="data.frame" && stop(gettextf("argument data=%s: not a data frame.", LABEL_data))
  any(is.na(match(c("diagage", "sex", "dy", "dm", "fy", "fm", "vitstat"), colnames(data)))) && stop(gettextf("argument data=%s: missing variable(s).", LABEL_data))
  any(is.na(match(data$sex, c("1", "2")))) && stop(gettextf("argument data=%s: variable 'sex' contains invalid values.", LABEL_data))
  any(is.na(match(data$vitstat, c("1", "2")))) && stop(gettextf("argument data=%s: variable 'vitstat' contains invalid values.", LABEL_data))
  for (var in c("diagage", "sex", "dy", "dm", "fy", "fm", "vitstat")) {
    any(is.na(suppressWarnings(data[[var]] <- as.integer(data[[var]])))) && stop(gettextf("argument data=%s: unable to coerce to integer.", LABEL_data))
  }


  #check and coerce argument k
  LABEL_k <- deparse(substitute(k))
  (missing(k) || is.null(k)) && stop(gettextf("missing argument k."))
  any(is.na(suppressWarnings(k <- as.integer(k)))) && stop(gettextf("argument k=%s: unable to coerce to integer.", LABEL_k))
  (length(k) > 1) && (k <- k[1]) && is.character(warning(gettextf("length of argument k=%s > 1: first element used.", LABEL_k),  call.=FALSE, immediate.=TRUE))
  (k < 1) && stop(gettextf("argument k=%s: not a positive integer.", LABEL_k))



  #check and coerce surv.m and surv.f
  LABEL_surv.m <- deparse(substitute(surv.m))
  LABEL_surv.f <- deparse(substitute(surv.f))
  LABEL_group <- deparse(substitute(group))
  (missing(surv.m) || is.null(surv.m)) && stop(gettextf("missing argument surv.m."))
  (missing(surv.f) || is.null(surv.f)) && stop(gettextf("missing argument surv.f."))

  #no group data provided, surv.m and surv.f must be data frames
  if (is.null(group)) {
    if(class(surv.m)!="data.frame") {
      stop(gettextf("argument surv.m=%s: not a data frame.", LABEL_surv.m))
    } else {
      #create dummy list
      surv.m <- list("1"=surv.m)
    }
    if(class(surv.f)!="data.frame") {
      stop(gettextf("argument surv.f=%s: not a data frame.", LABEL_surv.f))
    } else {
      #create dummy list
      surv.f <- list("1"=surv.f)
    }
    #create vector of group variables
    group <- rep(1, nrow(data))

  #group data were provided
  } else {

    (length(group)!=nrow(data)) && stop(gettextf("argument group=%s: invalid length.", LABEL_group))
    (any(is.na(group))) && stop(gettextf("argument group=%s: missing values.", LABEL_group))
    (!(is.numeric(group) || is.character(group))) && stop(gettextf("argument group=%s: invalid type.", LABEL_group))

    #check, whether lists were provided
    (class(surv.m)!="list") && stop(gettextf("argument surv.m=%s: not a list.", LABEL_surv.m))
    (class(surv.f)!="list") && stop(gettextf("argument surv.f=%s: not a list.", LABEL_surv.f))

    #if list elements are unnamed, assign integers
    (is.null(names(surv.m))) && (names(surv.m) <- 1:length(surv.m))
    (is.null(names(surv.f))) && (names(surv.f) <- 1:length(surv.f))

    any(is.na(match(unique(group), names(surv.m)))) && stop(gettextf("argument surv.m=%s: list not corresponding to values of argument group=%s.", LABEL_surv.m, LABEL_group))
    any(is.na(match(unique(group), names(surv.f)))) && stop(gettextf("argument surv.f=%s: list not corresponding to values of argument group=%s.", LABEL_surv.f, LABEL_group))
  }


  #check surv.m
  #check colnames and coerce values
  for (i in 1:length(surv.m)) {
    (class(surv.m[[i]])!="data.frame") && stop(gettextf("argument surv.m=%s: not a (list of) data frame(s).", LABEL_surv.m))
    (length(grep("^[[:digit:]]{4}$", colnames(surv.m[[i]])))!=ncol(surv.m[[i]])) &&  stop(gettextf("argument surv.m=%s: invalid column name(s).", LABEL_surv.m))
    for (col in colnames(surv.m[[i]])) {
      any(is.na(suppressWarnings(surv.m[[i]][[col]] <- as.numeric(surv.m[[i]][[col]])))) && stop(gettextf("argument surv.m=%s: unable to coerce to numeric.", LABEL_surv.m))
    }
  }

  #check surv.m
  #check colnames and coerce values
  for (i in 1:length(surv.f)) {
    (class(surv.f[[i]])!="data.frame") && stop(gettextf("argument surv.f=%s: not a (list of) data frame(s).", LABEL_surv.f))
    (length(grep("^[[:digit:]]{4}$", colnames(surv.f[[i]])))!=ncol(surv.f[[i]])) && stop(gettextf("argument surv.f=%s: invalid column name.", LABEL_surv.f))
    for (col in colnames(surv.f[[i]])) {
      any(is.na(suppressWarnings(surv.f[[i]][[col]] <- as.numeric(surv.f[[i]][[col]])))) && stop(gettextf("argument surv.f=%s: unable to coerce to numeric.", LABEL_surv.f))
    }
  }



  #check arguments diagbeg, perbeg and perend
  LABEL_diagbeg <- deparse(substitute(diagbeg))
  (missing(diagbeg) || is.null(diagbeg)) && stop("argument diagbeg is missing.")
  any(is.na(suppressWarnings(diagbeg <- as.integer(diagbeg)))) && stop(gettextf("argument diagbeg=%s: unable to coerce to integer.", LABEL_diagbeg))
  (length(diagbeg)>1) && (diagbeg <- diagbeg[1]) && is.character(warning(gettextf("argument diagbeg=%s: first element used.", LABEL_diagbeg), call.=FALSE, immediate.=TRUE))

  LABEL_perbeg <- deparse(substitute(perbeg))
  (missing(perbeg) || is.null(perbeg)) && stop("missing argument perbeg.")
  any(is.na(suppressWarnings(perbeg <- as.integer(perbeg)))) && stop(gettextf("argument perbeg=%s: unable to coerce to integer.", LABEL_perbeg))
  (length(perbeg) > 1) && (perbeg <- perbeg[1]) && is.character(warning(gettextf("length of argument perbeg=%s > 1: first element used.", LABEL_perbeg),  call.=FALSE, immediate.=TRUE))

  LABEL_perend <- deparse(substitute(perend))
  (missing(perend) || is.null(perend)) && stop("missing argument perend.")
  any(is.na(suppressWarnings(perend <- as.integer(perend)))) && stop(gettextf("argument perend=%s: unable to to coerce to integer.", LABEL_perend))
  (length(perend) > 1) && (perend <- perend[1]) && is.character(warning(gettextf("length of argument perend=%s > 1: first element used.", LABEL_perend), call.=FALSE, immediate.=TRUE))

  (perend < perbeg) && stop(gettextf("arguments perbeg=%s and perend=%s: invalid specification of the period of analysis", LABEL_perbeg, LABEL_perend))
  (perbeg <= diagbeg) && stop(gettextf("argument diagbeg=%s: invalid specification of the period of analysis.", LABEL_diagbeg))


  #check whether surv.m and surv.f provide survival probabilites for the entire period specified
  for (i in 1:length(surv.m)) {
    (min(as.integer(colnames(surv.m[[i]])))>min(diagbeg, perbeg)) && stop(gettextf("argument surv.m=%s: does not provide survival probabilities for the entire period of analysis.", LABEL_surv.m))
  }
  for (i in 1:length(surv.f)) {
    (min(as.integer(colnames(surv.f[[i]])))>min(diagbeg, perbeg)) && stop(gettextf("argument surv.f=%s: does not provide survival probabilities for the entire period of analysis.", LABEL_surv.f))
  }


  #check and coerce argument agedist
  if (!is.null(agedist)) {
    LABEL_agedist <- deparse(substitute(agedist))
    (class(agedist)=="data.frame") || stop(gettextf("argument agedist=%s: not a data frame.", LABEL_agedist))
    any(is.na(match(c("label", "agecat", "s"), colnames(agedist)))) && stop(gettextf("argument agedist=%s: missing variable(s).", LABEL_agedist))
    any(is.na(suppressWarnings(agedist$agecat <- as.integer(agedist$agecat)))) && stop(gettextf("argument agedist=%s: unable to coerce variable 'agecat' to integer.", LABEL_agedist))
    any(is.na(suppressWarnings(agedist$s <- as.numeric(agedist$s)))) && stop(gettextf("argument agedist=%s: unable to coerce variable 's' to numeric.", LABEL_agedist))
  }


  #duration of period in calendar years
  duration <- perend - diagbeg + 1

  #compute age-group specific weights required for age-adjusted survival
  n_agr <- 1

  #information about age group is passed to C routines (default 1, if crude
  #survival is computed)
  agr <- rep.int(1, nrow(data))

  #g := ratio of age-group specific proportions in standard population and
  #study cohort (default 1, if no age adjustment is done)
  #this weight is derived for all patients combined

  g <- 1
  if (!is.null(agedist)) {
    n_agr <- nrow(agedist)
    agr <- findInterval(data$diagage, vec=agedist$agecat, rightmost.closed=FALSE)
    r <- rep.int(0, n_agr)
    t <- table(agr)/length(data$diagage)*100

    #check if the data contain patients with age below the lowest age category
    if(any(rownames(t)==0)) {
      stop(gettextf("argument data=%s: include patients with age below the lowest age category.", LABEL_data))
    }

    r[as.integer(rownames(t))] <- t
    g <- as.vector(agedist$s/r)

    if(any(r==0)) {
      stop(gettextf("argument data=%s: does not include patients from all age categories.", LABEL_data))
    }

  }

  #takes return value of c routine
  rval <- NULL


  #these variables are used to sum up group specific values
  observations <- 0
  person.years <- obs.deaths <- exp.deaths <- 0
  sperl <- sperd <- sperde <- 0


  for (i in unique(group)) {

    #assign group specific data
    data_ <- data[which(group==i),]
    agr_ <- agr[which(group==i)]

    surv.m_ <- surv.m[[i]]
    surv.f_ <- surv.f[[i]]

    #computation of person years, observed and expected deaths for each combination
    #of follow-up year and calendar year of period
    rval <- .Call("hperiod_C", data_$diagage, data_$sex, data_$dy, data_$dm,
       data_$fy, data_$fm, data_$vitstat, k, surv.m_, surv.f_, diagbeg, perbeg, perend, agr_, n_agr)

    #sum observations accross all groups
    observations <- observations + rval$observations


    #calculate (weighted) sums of person years, observed and expected deaths for each follow up year
    #sum these sums accross all groups
    #sperl, sperd, sperde, sperle contain weighted sums
    sperl_ <- addmargins(rval$person.years,  2)[,duration+1,] * rep(g, each=k)
    if (!is.null(dim(sperl_))) {
      sperl_ <- rowSums(sperl_)
    }
    sperl <- sperl + sperl_

    sperd_ <- addmargins(rval$obs.deaths,  2)[,duration+1,] * rep(g, each=k)
    if (!is.null(dim(sperd_))) {
      sperd_ <- rowSums(sperd_)
    }
    sperd <- sperd + sperd_

    sperde_ <- addmargins(rval$exp.deaths,  2)[,duration+1,] * rep(g, each=k)
    if (!is.null(dim(sperde_))) {
      sperde_ <- rowSums(sperde_)
    }
    sperde <- sperde + sperde_


    #sum accross all groups
    person.years <- person.years + rval$person.years
    obs.deaths <- obs.deaths + rval$obs.deaths
    exp.deaths <- exp.deaths + rval$exp.deaths

  }
  
  
  #initialize vectors of survival proportions (abssur, condabssur, expsur, condexpsur, relsur and condrelsur)
  #and standard errors (vari and abserr, condabserr, relerr, condrelerr)
  abssur <- condabssur <- expsur <- condexpsur <- relsur <- condrelsur <- rep.int(NA, k)
  
  for (i in 1:k) {

    #derive no estimate, if no follow-up information was provided for
    #follow-up year i
    if (sperl[i]>0) {

      condabssur[i] <- 1 - (sperd[i]/sperl[i])
      
      if (sperd[i]>0 & sperd[i]>=sperl[i]) {
        abssur[i] <- 0
      } else if (i==1) {
        abssur[i] <- condabssur[i]
      } else if (abssur[i-1]==0) {
        abssur[i] <- 0
      } else {
    	  abssur[i]  <- abssur[i-1] * condabssur[i]
      }

      #expected survival according to Ederer II method
      condexpsur[i] <- 1 - (sperde[i]/sperl[i])

      if (i==1) {
        expsur[i] <- condexpsur[i]
      } else {
        expsur[i] <- expsur[i-1] * condexpsur[i]
      }

      condrelsur[i] <- condabssur[i]/condexpsur[i]
      
  		if (abssur[i]==0) {
  		  relsur[i] <- 0
   		} else {
   		  relsur[i] <- abssur[i]/expsur[i]
   		}
    } else {
      abssur[i] <- relsur[i] <- 0
    }
  }


  #compute standard errors of survival estimates for each follow-up year (1<=i<=k)

  #initialize required vectors
  vari <- abserr <- condabserr <- relerr <- condrelerr <- term <- rep(NA, k)

  if (is.null(agedist)) {

    # Greenwood's formula of standard error absolute survival
    # abssur * sqrt ( sum ( d_i / (l_i * (l_i - d_i)) ) )
    term <- rep.int(0, k)
    for (i in 1:k) {

      #derive no estimate if sperl[i]==0
  		if (abssur[i]==0 | sperl[i]==0) {
  			term[i] <- NA

      #derive follow-up year specific and cumulative estimates
  		} else {
        term[i] <- sperd[i]/(sperl[i]*(sperl[i] - sperd[i]))
      }

      condabserr[i] <- condabssur[i]*sqrt(term[i])
      abserr[i] <- abssur[i]*sqrt(sum(term[1:i]))
      condrelerr[i] <- condabserr[i]/condexpsur[i]
      relerr[i] <- abserr[i]/expsur[i]

    }
    
  } else {
    #standard errors using a generalized Greenwood formula according to [12]




    #standard errors according to [11]
    #generalized Greenwood formula for
    #variance of conditional absolute survival in follow-up year i
    #p_i^2 * sum ( g_a^2 * (d_a_i / py_a_i) * (py_a_i - d_a_i)) / ( sum ( g_a * (py_a_i - d_a_i) ) )^2
    #
    term <- rep.int(0, k)
    for (i in 1:k) {

    #derive no estimate of standard error, if no follow-up was observed during follup-year i
    if(sum(rval$person.years[i,,])==0) {
      term[i] <- NA

    #derive estimate
    #note: sum(rval$....[i,,j]) is an unweighted sum of person years or  deaths in age category j and follow-up year i
    } else {
      sum1 <- sum2 <- 0

      for (j in 1:n_agr) {

        #if no person years were observed in age category j,
        #then by definition survival in age category j and follow-up year is equals zero
        #no contribution to sum1 and sum2

        if (sum(rval$person.years[i,,j]) > 0) {
          sum1 <- sum1 + g[j]^2 * (sum(rval$obs.deaths[i,,j]) / sum(rval$person.years[i,,j])) * (sum(rval$person.years[i,,j]) - sum(rval$obs.deaths[i,,j]))
          sum2 <- sum2 + g[j] * (sum(rval$person.years[i,,j]) - sum(rval$obs.deaths[i,,j]))
          }
      }

      term[i] <- sum1/(sum2^2)

    }

      condabserr[i] <- condabssur[i] * sqrt(term[i])
      abserr[i] <- abssur[i] * sqrt(sum(term[1:i]))
      condrelerr[i] <- condabserr[i]/condexpsur[i]
      relerr[i] <- abserr[i]/expsur[i]
    }
  }



  #format/arrange return values
  if (n_agr > 1) {
    dimnames_ <- list(1:k, diagbeg:perend, agedist$label)
  } else {
    dimnames_ <- list(1:k, diagbeg:perend, "overall")
  }

  rval$observations <- observations
  rval$exp.deaths <- exp.deaths
  rval$obs.deaths <- obs.deaths
  rval$person.years <- person.years

  dimnames(rval$exp.deaths) <- dimnames(rval$obs.deaths) <- dimnames(rval$person.years) <- dimnames_

  #initialize period.object instance
  rval <- c(rval, list(abs.surv=abssur*100,
                        abs.surv.stderr=abserr*100,
                        cond.abs.surv=condabssur*100,
                        cond.abs.surv.stderr=condabserr*100,
                        rel.surv=relsur*100,
                        rel.surv.stderr=relerr*100,
                        cond.rel.surv=condrelsur*100,
                        cond.rel.surv.stderr=condrelerr*100,
                        method="edererII"),
                        call=sys.call(),
                        version="1.0-6")

  if (!is.null(agedist)) {
    rval <- c(rval, list(age.adjusted=TRUE, w=g))
  } else {
    rval <- c(rval, list(age.adjusted=FALSE, w=NULL))
  }

  class(rval) <- "period"
  return(rval)
}

