\encoding{latin1}
\name{hybridperiod}
\alias{hybridperiod}
\title{Computation of Absolute and Relative Survival Estimates
  Based on a Hybrid Period Analysis Approach}
\description{
  Computation of absolute and relative survival (both follow-up
  year-specific and cumulative estimates) based on an adapted
  approach if \sQuote{classical} period analysis is not possible.
}
\usage{hybridperiod(data, k, surv.m, surv.f, diagbeg, perbeg,
    perend, agedist=NULL, group=NULL)
}
\arguments{
  \item{data}{data frame consisting of the numeric variables\cr
      \code{sex} - sex (\code{1}=male, \code{2}=female),\cr
      \code{diagage} - age at diagnosis in years,\cr
      \code{dm} - month of diagnosis,\cr
      \code{dy} - calendar year of diagnosis,\cr
      \code{fm} - month of end of follow-up,\cr
      \code{fy} - calendar year of end of follow-up,\cr
      \code{vitstat} - vital status at end of follow-up
        (\code{1}=alive, \code{2}=dead).}

  \item{k}{follow-up time in years (numeric; e.g. \code{k=5}
    for computation of 5-year survival estimates).}
    
  \item{surv.m}{data frame(s) of age- and calendar year-specific
    survival probabilities of males in the underlying population
    made up of numeric vectors of length 100 providing 1-year
    survival probabilities from age 0 to 99 years (the function
    however accepts survival probabilities up to any other age
    limit).  Survival probabilities of a particular calendar year
    are accessed via the vectors' names (e. g. if the data frame
    consists of three vectors named \dQuote{2000}, \dQuote{2003}
    and \dQuote{2004}, the vector named \dQuote{2000} will be used to
    derive survival probabilities of the calendar years 2000-02,
    \dQuote{2003} for survival probabilities in 2003 and
    \dQuote{2004} for survival probabilities in 2004 and later
    (therefore, each data frame requires a column with name <=
    \code{perbeg}. The conditional 1-year survival of a man aged
    49 years in 2001 may then be obtained as
    \code{surv.m[["2000"]][50]})).\cr
    It is also possible to provide survival probabilities for
    different subgroups of patients (e. g. according to region or
    ethnic origin).  Then \code{surv.m} must be provided as a named
    list (of data frames as specified before) along with argument
    \code{group}, which contains for each observation the name of
    the respective element/data frame of survival probabilities
    of \code{surv.m}. See \code{\link{surv.probs.males}} for further
    details.}

  \item{surv.f}{data frame(s) of age- and calendar year-specific
    survival probabilities of females in the underlying
    population (see argument \code{surv.m} and
    \code{\link{surv.probs.females}} for further details).}


  \item{diagbeg}{first calendar year of the period of analysis
    relevant for patients who contribute all observed survival to
    the estimation (\code{diagbeg <= perbeg}; numeric).}
    
  \item{perbeg}{first calender year of the period of analysis
    relevant for patients diagnosed prior to \code{diagbeg} (numeric).}
    
  \item{perend}{last calendar year of period of analysis (numeric).}
    
  \item{agedist}{data frame specifying age categories and
    proportions of a reference population of cancer patients for
    computation of age standardized survival according to the
    \sQuote{Brenner} approach (see \code{\link{cancer.pop}}
    and the Details section below for further information). By
    default, no age adjustment is done.}

  \item{group}{vector specifying the group membership of each
    patient (integer or character vector).  If provided, the
    argument must specify the name of the corresponding data
    frame of subgroup specific survival probabilities (a list
    element of \code{surv.m} and \code{surv.f}, respectively) for
    each observation included in argument \code{data}.  By default, except for
    sex, no further stratification of patients with regard to
    overall mortality is done.}
}

\details{
  The period analysis approach provides more up-to-date
  estimates of long term survival compared to traditional
  methods. [1, 2]  It exclusively reflects the survival
  experience of patients within a specified calendar period
  (e.g. the most recent calendar years, for which mortality
  follow-up is available).  Several evaluations demonstrated that
  the method enables the early detection of changes in survival
  trends and better predicts long term survival of concurrently
  diagnosed and treated patients than survival estimates derived
  by traditional methods. [3-6]

  Under certain circumstances \sQuote{classical} period analysis
  is not possible (e. g. if recording of incident cases is
  delayed by several years whereas more recent information on
  the vital status of the patients is available).  Based
  on an adapted approach, \code{hybridperiod} provides long
  term survival estimates that are as up-to-date as possible.
  [7, 8]

  Based on life-table methodology, point estimates of conditional
  absolute and relative survival for 1-year intervals after
  diagnosis as well as cumulative survival estimates are
  calculated. [9]  The \sQuote{Ederer II} method is implemented
  for computation of relative survival. [10]  If expected survival
  for patients above the age of 99 years is needed, the survival
  probability for 99 year old patients will be used (since the
  proportion of these patients is very small usually, this
  simplification will not in any meaningful way  influence the
  observed estimates of relative survival).

  The estimation of age-standardized survival is based on a
  recently proposed approach (\sQuote{Brenner} method) [11] which
  improves the methodological consistency of age-adjusted
  survival estimation and which is less likely to fail due to
  sparse age-specific data (e. g. in older ages) than traditional
  methods.  This analysis approach uses weightened individual data for
  computation instead of combining weighted age-specific survival
  estimates. The program fails, if there are age-categories
  without any observation.

  Standard errors of the survival estimates are calculated
  according to Greenwood's method. [12]  For age-standardized
  survival estimates derived with the \sQuote{Brenner} method [11],
  a generalized Greenwood formula is used. [13]
  
  The table below illustrates how survival observed for patients
  diagnosed between 1996 and 2003 would be used in a hybrid
  period analysis to obtain the most up-to-date 5-year survival
  estimates using a database of cancer patients with follow-up
  until end of 2005.  The numbers within the cells indicate the
  years following diagnosis.

\preformatted{
   year                    year of follow-up
    of  |
   diag | 1996 1997 1998 1999 2000 2001 2002 | 2003 2004 2005
--------|------------------------------------|----------------
   1996 |    1  1/2  2/3  3/4  4/5    5      |
   1997 |         1  1/2  2/3  3/4  4/5    5 |
   1998 |              1  1/2  2/3  3/4  4/5 |    5
   1999 |                   1  1/2  2/3  3/4 |  4/5    5
   2000 |                        1  1/2  2/3 |  3/4  4/5    5
--------|------------------------------------|----------------
   2001 |                             1  1/2 |  2/3  3/4  4/5
   2002 |                                  1 |  1/2  2/3  3/4
   2003 |                                    |    1  1/2  2/3
        |                                    |
}

  The estimates are derived from survival observed during the
  calendar period between 2003 and 2005 for patients diagnosed
  up to 2000 (cells in the upper right section) and the complete
  survival observed for patients diagnosed from 2001 onwards
  (cells in the lower sections; parameter specification:
  \code{diagbeg=2001}, \code{perbeg=2003} and \code{perend=2005}).
}
\value{An object of class \code{\link{period.object}}.}

\author{Bernd Holleczek}

\note{This function implements and extends existing SAS makros
  for period analysis which are available at
  \url{http://www-old.imbe.med.uni-erlangen.de/issan/SAS/period/period.htm}
  (accessed December 20, 2011). [7, 8]
}


\references{
[1] Brenner H, Gefeller O (1996) An alternative approach to
  monitoring cancer patient survival. \emph{Cancer} 78:
  2004--2010.

[2] Brenner H, Gefeller O (1997) Deriving more up-to-date
  estimates of long-term patient survival.
  \emph{J Clin Epidemiol} 50: 211--216.

[3] Brenner H, Hakulinen T (2002) Advanced detection of time
  trends in long-term cancer patient survival: experience from
  50 years of cancer registration in Finland.
  \emph{Am J Epidemiol} 156: 566--577.

[4] Brenner H, Hakulinen T (2002) Up-to-date long-term survival
  curves of patients with cancer by period analysis.
  \emph{J Clin Oncol} 20: 826--832.

[5] Brenner H, Sderman B, Hakulinen T (2002) Use of period
  analysis for providing more up-to-date estimates of long-term
  survival rates: empirical evaluation among 370 000 cancer
  patients in Finland. \emph{Int J Epidemiol} 31: 456--462.

[6] Talback M, Stenbeck M, Rosen M (2004) Up-to-date long-term
  survival of cancer patients: an evaluation of period analysis
  on Swedish Cancer Registry data. \emph{Eur J Cancer} 40: 1361--1372.

[7] Brenner H, Gefeller O, Hakulinen T (2002) A computer program
  for period analysis of cancer patient survival.
  \emph{Eur J Cancer} 38: 690--695.

[8] Brenner H, Rachet B (2004) Hybrid analysis for up-to-date
  long-term survival rates in cancer registries with delayed
  recording of incident cases.
  \emph{Eur J Cancer} 40: 2494--2501.
  
[9] Esteve J, Benhamou E, Raymond L (1994) \emph{Statistical
  Methods in Cancer Research. Volume IV. Descriptive Epidemiology.
  IARC Scientific Publications No. 128}. Lyon: International
  Agency for Research on Cancer.

[10] Ederer F, Heise H (1959)
  \emph{Instructions to IBM 650 programmers in processing survival
  computations}. Bethesda, MD: National Cancer Institute.

[11] Brenner H, Arndt V, Gefeller O, Hakulinen T (2004)
  An alternative approach to age adjustment of cancer survival
  rates. \emph{Eur J Cancer} 40: 2317--2322.

[12] Greenwood M (1926) \emph{The natural duration of cancer}.
  In: Reports on Public Health and Medical Subjects. London: Her
  Majesty's Stationery Office. 1--26.


[13] Pokhrel A, Dyba T, Hakulinen T (2008) A Greenwood formula
  for standard error of the age-standardized relative survival
  ratio. \emph{Eur J Cancer} 44: 441--447.
}

\seealso{\code{\link{stomach}}, \code{\link{surv.probs.males}},
  \code{\link{surv.probs.females}}, \code{\link{cancer.pop}},
  \code{\link{period}}, \code{\link{period.object}}.
}

\examples{
## load data of stomach cancer patients

data(stomach)


## load age-specific survival proportions of males and females
## in Germany during 1996-2005

data(surv.probs.males)
data(surv.probs.females)


## hybrid analysis
##
## compute suvival estimates with hybrid period analysis with
## best timeliness available if 'classical' period analysis is
## not possible (according to the example in the Details section)
##
## example:
## delay of incidence recording by two years (cases diagnosed
## until end of 2003) whereas follow-up information is available
## until end of 2005

hybrid.period.result <- hybridperiod(subset(stomach, dy<=2003),
    5, surv.probs.males, surv.probs.females, 2001, 2003, 2005)

print(hybrid.period.result)
}
\keyword{survival}

