\encoding{latin1}
\name{period}
\alias{period}

\title{Computation of Absolute and Relative Survival Estimates
  Based on the Period Analysis Approach}
\description{Computation of absolute and relative survival (both
  follow-up year-specific and cumulative estimates) based on the
  period analysis approach.
}
\usage{period(data, k, surv.m, surv.f, perbeg, perend,
    method=c("edererII", "hakulinen"), agedist=NULL, group=NULL)
}
\arguments{
  \item{data}{data frame consisting of the numeric variables\cr
      \code{sex} - sex (\code{1}=male, \code{2}=female),\cr
      \code{diagage} - age at diagnosis in years,\cr
      \code{dm} - month of diagnosis,\cr
      \code{dy} - calendar year of diagnosis,\cr
      \code{fm} - month of end of follow-up,\cr
      \code{fy} - calendar year of end of follow-up,\cr
      \code{vitstat} - vital status at end of follow-up
        (\code{1}=alive, \code{2}=dead).}

  \item{k}{follow-up time in years (numeric; e.g. \code{k=5}
    for computation of 5-year survival estimates).}

  \item{surv.m}{data frame(s) of age- and calendar year-specific
    survival probabilities of males in the underlying population
    made up of numeric vectors of length 100 providing 1-year
    survival probabilities from age 0 to 99 years (the function
    however accepts survival probabilities up to any other age
    limit).  Survival probabilities of a particular calendar year
    are accessed via the vectors' names (e. g. if the data frame
    consists of three vectors named \dQuote{2000}, \dQuote{2003} and
    \dQuote{2004}, the vector named \dQuote{2000} will be used to
    derive survival probabilities of the calendar years 2000-02,
    \dQuote{2003} for survival in 2003 and \dQuote{2004} for
    survival in 2004 and later (therefore, each data frame requires
    a column with name <= \code{perbeg}. The conditional 1-year
    survival of a man aged 49 years in 2001 may be obtained as
    \code{surv.m[["2000"]][50]})).\cr
    It is also possible to provide survival probabilities for
    different subgroups of patients (e. g. according to region or
    ethnic origin).  Then \code{surv.m} must be provided as a named
    list (of data frames as specified before) along with argument
    \code{group}, which contains for each observation the name of
    the respective element/data frame of survival probabilities
    of \code{surv.m}. See \code{\link{surv.probs.males}} for further
    details.}

  \item{surv.f}{data frame(s) of age- and calendar year-specific
    survival probabilities of females in the underlying
    population (see argument \code{surv.m} and
    \code{\link{surv.probs.females}} for further details).}
    
  \item{perbeg}{first calender year of the period of analysis
    (numeric)}
    
  \item{perend}{last calendar year of the period of analysis
    (numeric)}
    
  \item{method}{method for calculation of relative survival
    (character vector; either \code{edererII} (default) or
    \code{hakulinen}).}
    
  \item{agedist}{data frame specifying age categories and
    proportions of a reference population of cancer patients for
    computation of age standardized survival according to the
    \sQuote{Brenner} approach (see \code{\link{cancer.pop}}
    and the Details section below for further information). By
    default, no age adjustment is done.}
    
  \item{group}{vector specifying the group membership of each
    patient (integer or character vector).  If provided, the
    argument must specify the name of the corresponding data
    frame of subgroup specific survival probabilities (a list
    element of \code{surv.m} and \code{surv.f}, respectively) for
    each observation included in argument \code{data}.  By default,
    except for sex, no further stratification of patients with
    regard to overall mortality is done.}
}

\details{The period analysis approach provides more up-to-date
  estimates of long term survival compared to traditional methods.
  [1, 2]  It exclusively reflects the survival experience of
  patients within a specified calendar period (e.g. the most
  recent calendar years, for which mortality follow-up is
  available).  Several evaluations demonstrated that the method
  enables the early detection of changes in survival trends and
  better predicts long term survival of concurrently diagnosed
  and treated patients than survival estimates derived by
  traditional methods. [3--6]
  
  Based on life-table methodology, point estimates of conditional
  absolute and relative survival for 1-year intervals after
  diagnosis as well as cumulative survival estimates are
  calculated. [7]  There are two methods implemented for computation
  of relative survival: the \sQuote{Ederer II} method [8] and
  the \sQuote{Hakulinen} method. [9]  Until recently, the \sQuote{Hakulinen} method was recommended
  for longer follow-up times [10], however, recent research now propagates
  the \sQuote{Ederer II} method for deriving relative survival estimates. [11]

  The implemented estimation of age-standardized survival is based
  on a recently proposed approach (\sQuote{Brenner} method) [12] which
  improves the  methodological consistency of age-adjusted survival
  estimation and which is less likely to fail due to sparse age-specific
  data (e.g. in older ages) than traditional methods.  This
  analysis approach uses weightened individual data for
  computation instead of combining weighted age-specific survival
  estimates. The program fails, if there are age-categories
  without any observation.

  Standard errors of the survival estimates are calculated
  according to Greenwood's method. [13]  For age-standardized
  survival estimates derived with the \sQuote{Brenner} method [12],
  a generalized Greenwood formula is used. [14]
  
  The computation of age standardized survival estimates based
  on the classical approach (using weightened age specidic survival
  estimates) [7] is also illustraded in the Examples section.

  Besides the calculation of survival estimates according to the
  period analysis approach, other analysis strategies as
  traditional cohort based or complete approaches may be
  performed as well. [15]

  The table below illustrates how survival observed for patients
  diagnosed between 1996 and 2005 would be used to obtain the most
  up-to-date 5-year survival estimates by cohort and period
  analysis with a database including patients diagnosed and
  followed up until end of 2005 (using data of five subsequent
  years of diagnosis and a period of analysis of 5 calendar years,
  respectively).  The numbers within the cells indicate
  the years following diagnosis.
  
  \preformatted{
   year                    year of follow-up
    of  |
   diag | 1996 1997 1998 1999 2000 | 2001 2002 2003 2004 2005
--------|--------------------------|--------------------------
   1996 |    1  1/2  2/3  3/4  4/5 |    5
   1997 |         1  1/2  2/3  3/4 |  4/5    5
   1998 |              1  1/2  2/3 |  3/4  4/5    5
   1999 |                   1  1/2 |  2/3  3/4  4/5    5
   2000 |                        1 |  1/2  2/3  3/4  4/5    5
--------|--------------------------|--------------------------
   2001 |                          |    1  1/2  2/3  3/4  4/5
   2002 |                          |         1  1/2  2/3  3/4
   2003 |                          |              1  1/2  2/3
   2004 |                          |                   1  1/2
   2005 |                          |                        1
        |                          |
  }
  
  The cohort estimates are derived from complete survival
  observed for patients diagnosed in 1996-2000 (cells in the
  upper sections), the period estimates are based on survival
  observed in the calendar period 2001-2005 (cells in the
  right sections).
  
  Recently introduced model based period analysis uses the
  framework of generalized linear models and has extended
  available period analysis methodology. [16]  Model based period
  analysis has shown to provide accurate and up-to-date survival
  estimates with increased precision.  It further allows to
  assess and to account for effects of additional covariates such
  as age, stage of disease, calendar time or the patient's region
  or ethnic origin. In the Examples section, two examples are provided to
  demonstrate how model based period analysis may be used for
  estimating age group-specific 5-year relative survival and for
  testing for trends over time in 5-year relative survival of cancer
  patients.

}
\value{An object of class \code{\link{period.object}}.
}

\references{
[1] Brenner H, Gefeller O (1996) An alternative approach to
  monitoring cancer patient survival. \emph{Cancer} 78: 2004--2010.
  
[2] Brenner H, Gefeller O (1997) Deriving more up-to-date
  estimates of long-term patient survival. \emph{J Clin Epidemiol}
  50: 211--216.
  
[3] Brenner H, Hakulinen T (2002) Advanced detection of time
  trends in long-term cancer patient survival: experience from 50
  years of cancer registration in Finland. \emph{Am J Epidemiol}
  156: 566--577.
  
[4] Brenner H, Hakulinen T (2002) Up-to-date long-term survival
  curves of patients with cancer by period analysis.
  \emph{J Clin Oncol} 20: 826--832.
  
[5] Brenner H, Sderman B, Hakulinen T (2002) Use of period
  analysis for providing more up-to-date estimates of long-term
  survival rates: empirical evaluation among 370 000 cancer
  patients in Finland. \emph{Int J Epidemiol} 31: 456--462.

[6] Talback M, Stenbeck M, Rosen M (2004) Up-to-date long-term
  survival of cancer patients: an evaluation of period analysis
  on Swedish Cancer Registry data. \emph{Eur J Cancer} 40: 1361--1372.
  
[7] Esteve J, Benhamou E, Raymond L (1994) \emph{Statistical
  Methods in Cancer Research. Volume IV. Descriptive Epidemiology.
  IARC Scientific Publications No. 128}. Lyon: International
  Agency for Research on Cancer.
  
[8] Ederer F, Heise H (1959)
  \emph{Instructions to IBM 650 programmers in processing survival
  computations}. Bethesda, MD: National Cancer Institute.
  
[9] Hakulinen T (1982) Cancer survival corrected for heterogeneity
  in patient withdrawal. \emph{Biometrics} 38: 933--942.
  
[10] Voutilainen ET, Dickman PW, Hakulinen T (2000) \emph{SURV2:
  Relative Survival Analysis Program. Software Manual.} Helsinki:
  Finnish Cancer Registry.
  
[11] Hakulinen T, Sepp K, Lambert PC (2011) Choosing the relative
  survival method for cancer survival estimation. \emph{Eur J Cancer}
  47: 2202--2210.
  
[12] Brenner H, Arndt V, Gefeller O, Hakulinen T (2004) An
  alternative approach to age adjustment of cancer survival rates.
  \emph{Eur J Cancer} 40: 2317--2322.
  
[13] Greenwood M (1926) \emph{The natural duration of cancer}.
  In: Reports on Public Health and Medical Subjects. London: Her
  Majesty's Stationery Office. 1--26.
  
[14] Pokhrel A, Dyba T, Hakulinen T (2008) A Greenwood formula
  for standard error of the age-standardized relative survival
  ratio. \emph{Eur J Cancer} 44: 441--447.

[15] Brenner H, Gefeller O, Hakulinen T (2002) A computer program
  for period analysis of cancer patient survival.
  \emph{Eur J Cancer} 38: 690--695.
  
[16] Brenner H, Hakulinen T (2006) Up-to-date and precise estimates
  of cancer patient survival: model-based period analysis.
  \emph{Am J Epidemiol} 164: 689-696.
}

\author{Bernd Holleczek}

\note{This function implements and extends existing SAS makros
  for period analysis which are available at
  \url{http://www-old.imbe.med.uni-erlangen.de/issan/SAS/period/period.htm}
  (accessed December 21, 2011). [12]
}
\seealso{\code{\link{stomach}}, \code{\link{surv.probs.males}},
  \code{\link{surv.probs.females}}, \code{\link{cancer.pop}},
  \code{\link{hybridperiod}}, \code{\link{colorectum.df}},
  \code{\link{period.object}}.
}
\examples{

## load data of stomach cancer patients

data(stomach)


## load age-specific survival probabiliies of males and females
## in Germany in 1996-2005

data(surv.probs.males)
data(surv.probs.females)


## categorize patients according to age at diagnosis
##
## agr=0 for patients younger than 65 years
## agr=1 for patients 65 years and older

sto.agr <- data.frame(stomach,
    agr=findInterval(stomach$diagage, 65))


## cohort analysis
##
## compute age group-specific 5-year survival estimates of
## patients diagnosed with stomach cancer between 1996 and 2000
## (Ederer II method is used for relative survival)

cohort.result.agr0 <- period(subset(sto.agr,
    agr==0 & dy>=1996 & dy<=2000), 5, surv.probs.males,
    surv.probs.females, 1996, 2005, method="edererII")

print(cohort.result.agr0)

cohort.result.agr1 <- period(subset(sto.agr,
    agr==1 & dy>=1996 & dy<=2000), 5, surv.probs.males,
    surv.probs.females, 1996, 2005, method="edererII")

print(cohort.result.agr1)


## period analysis
##
## this analysis considers exclusively survival experience within
## the specified calendar period and provides best timeliness
## compared to cohort or complete analysis
##
## compute age group-specific 5-year survival based on survival
## observed during the calendar period 2001 to 2005 (Ederer II
## method is used for relative survival)

period.result.agr0 <- period(subset(sto.agr, agr==0), 5,
     surv.probs.males, surv.probs.females, 2001, 2005,
     method="edererII")

print(period.result.agr0)

period.result.agr1 <- period(subset(sto.agr, agr==1), 5,
    surv.probs.males, surv.probs.females, 2001, 2005,
    method="edererII")
    
print(period.result.agr1)


## complete analysis
##
## complete analysis uses the entire information of follow-up
## data that is observed in the specified calendar period for the
## calculation of survival estimates (in terms of timeliness
## these estimates are better than the cohort estimates, as the
## analysis additionally includes patients that were diagnosed
## more recently)
##
## compute 5-year survival based on all cases diagnosed since 1996
## (Ederer II method is used for relative survival)

compl.result.agr0 <- period(subset(sto.agr, agr==0), 5,
    surv.probs.males, surv.probs.females, 1996, 2005,
    method="edererII")

print(compl.result.agr0)

compl.result.agr1 <- period(subset(sto.agr, agr==1), 5,
    surv.probs.males, surv.probs.females, 1996, 2005,
    method="edererII")

print(compl.result.agr1)


## age-adjusted analysis ("Brenner approach" using
## weighted indivudual observations)
##

## load reference population of cancer patients
data(cancer.pop)

## compute age-adjusted 5-year survival estimates based on
## survival observed during 2001 to 2005 (Ederer II method is
## used for relative survival)

adj.result <- period(stomach, 5, surv.probs.males,
    surv.probs.females, 2001, 2005, method="edererII",
    agedist=cancer.pop)

print(adj.result)



## age-adjusted analysis ("classical" approach using weightened
## age specific survival estimates)
##

## compute age-adjusted 5-year relative survival estimates
## based on survival observed during 2001 to 2005 (Ederer II
## method is used for relative survival)
##

res_agr1 <- period(subset(colorectum.df, diagage < 45), 5,
  surv.probs.males, surv.probs.females, 2001, 2005)
  
res_agr2 <- period(subset(colorectum.df, diagage >= 45 & diagage < 55), 5,
  surv.probs.males, surv.probs.females, 2001, 2005)

res_agr3 <- period(subset(colorectum.df, diagage >= 55 & diagage < 65), 5,
  surv.probs.males, surv.probs.females, 2001, 2005)

res_agr4 <- period(subset(colorectum.df, diagage >= 65 & diagage < 75), 5,
  surv.probs.males, surv.probs.females, 2001, 2005)

res_agr5 <- period(subset(colorectum.df, diagage >= 75), 5,
  surv.probs.males, surv.probs.females, 2001, 2005)

s <- cancer.pop$s/100

adj.classical.result <- res_agr1$rel.surv*s[1] + res_agr2$rel.surv*s[2] +
  res_agr3$rel.surv*s[3] + res_agr4$rel.surv*s[4] + res_agr5$rel.surv*s[5]

print(adj.classical.result)

##
## derive standard error of age-adjusted survival estimate derived
## with classical approach
##

adj.classical.result.se <- sqrt(s[1]^2 * res_agr1$rel.surv.stderr^2 +
  s[2]^2 * res_agr2$rel.surv.stderr^2 + s[3]^2 * res_agr3$rel.surv.stderr^2 +
  s[4]^2 * res_agr4$rel.surv.stderr^2 + s[5]^2 * res_agr5$rel.surv.stderr^2)

print(adj.classical.result.se)



## Model based period analysis
##
## Two examples are presented to demonstrate how model based
## period analysis may be used for estimating age-group-specific
## 5-year RS of patients with colorectal cancer and for testing
## for trends over time in 5-year RS.
##
## Example 1: regression modelling to derive estimates of
## age-group-specific 5-year RS of patients with colorectal cancer
## for the calendar period 2003-05
##

## load sample data patients with colorectal cancer

data(colorectum.df)


## load sex amd age-specific survival probabilities
## in Germany in 1996-2005

data(surv.probs.males)
data(surv.probs.females)


## add variable with age category

colorectum.df$agr <- findInterval(colorectum.df$diagage, 75)


## print data of the first 3 patients

colorectum.df[1:3,]


## exclude death certificate only notified cases

colorectum.df <- subset(colorectum.df, dco=="N")


## derive vectors of model variables

y <- agr <- obs <- dstar <- at_risk <- NULL

for (agr_ in 0:1) {

    #run conventional period analysis
    res_ <- period(subset(colorectum.df, agr==agr_) , 5,
              surv.probs.males, surv.probs.females, 2003, 2005)

    py_ <- rowSums(res_$person.years[,,1])
    obs_ <- rowSums(res_$obs.deaths[,,1])
    exp_ <- rowSums(res_$exp.deaths[,,1])

    #append values
    y <- c(y, 1:5)
    agr <- c(agr, rep(agr_, 5))
    obs <- c(obs, obs_)
    dstar <- c(dstar, -(py_ - obs_/2)*log((py_ - exp_)/py_))
    at_risk <- c(at_risk, log(py_ - obs_/2))

}


## couple vectors of model variables in 'model1.df'

model1.df <- data.frame(y=as.factor(y), agr=as.factor(agr),
                            obs, dstar, at_risk)


## print first 3 observations included in the model

model1.df[1:3,]


## user defined link function

modperiod.link <- function(dstar) {
  structure(
    list(
      #link
      linkfun = function(mu) {log(mu - dstar)},
      #inverse link
      linkinv = function(eta) {exp(eta) + dstar},
      #derivative of the inverse link (dmu/deta)
      mu.eta = function(eta) {exp(eta)},

      #functions for domain checking
      valideta = function(eta) TRUE,
      validmu = function(mu) mu>dstar,

      name = "modperiod.link"),
    class = "link-glm")
}


## fit model

model1.fit <- glm(obs ~ y + agr - 1, offset=at_risk,
            family=poisson(modperiod.link(model1.df$dstar)), data=model1.df)


## print details of fitted model

summary(model1.fit)


## get model coefficients

y.coef <- model1.fit$coefficients[1:5]
agr.coef <- model1.fit$coefficients[6]


## derive conditional and cumulative relative survival (RS)

r.agr0 <- r.agr1 <- rep.int(0, 5)

for(j in 1:5) {
  r.agr0[j] <- exp(-exp(y.coef[j]))
  r.agr1[j] <- exp(-exp(y.coef[j] + agr.coef))
}

R.agr0 <- cumprod(r.agr0)*100
R.agr1 <- cumprod(r.agr1)*100


## get variance covariance matrix

var_ <- vcov(model1.fit)


## derive variance of 5-year RS of patients aged 15-74 years

A_agr0 <- R.agr0[5] * log(r.agr0)

var.R.agr0 <- sum(A_agr0^2 *
      c(var_[1,1], var_[2,2], var_[3, 3], var_[4, 4], var_[5, 5]))

term <- 0
for (k in 1:5) {
  for (l in 1:5) {
    if (k!=l) {
      term <- term + A_agr0[k] * A_agr0[l] * var_[k, l]
    }
  }
}

var.R.agr0 <- var.R.agr0 + term


## derive variance of 5-year RS of patients aged 75+ years

A_agr1 <- R.agr1[5] * log(r.agr1)
B_agr1 <- R.agr1[5] * sum(log(r.agr1[1:5]))

var.R.agr1 <- sum(A_agr1^2 * c(var_[1,1], var_[2,2], var_[3, 3],
      var_[4, 4], var_[5, 5])) +
      (B_agr1^2 * var_[6, 6] + 2 * sum(B_agr1 * A_agr1 * c(var_[1:5, 6])))

term <- 0
for (k in 1:5) {
  for (l in 1:5) {
    if (k!=l) {
      term <- term + A_agr1[k]*A_agr1[l] * var_[k, l]
    }
  }
}


var.R.agr1 <- var.R.agr1 + term


## print modelled estimate of 5-year RS of patients aged 15-74 years
## and its standard error

R.agr0[5]
sqrt(var.R.agr0)


## print standard error of 5-year RS of patients aged 75+ years
## and its standard error

R.agr1[5]
sqrt(var.R.agr1)


## print results of 'classical' period analysis

period(subset(colorectum.df, agr=="0"), 5,
        surv.probs.males, surv.probs.females, 2003, 2005)
        
period(subset(colorectum.df, agr=="1"), 5,
        surv.probs.males, surv.probs.females, 2003, 2005)


## Example 2: regression modelling for testing for trend over time
## in 5-year RS of patients with colorectal cancer during the
## calendar period 1997-2005
##

## add variable 'agr' with age category

colorectum.df$agr <- findInterval(
                        colorectum.df$diagage, c(45, 55, 65, 75))


## obtain vectors of model variables

y <- cp <- agr <- obs <- dstar <- at_risk <- NULL

for (cp_ in 1:3) {
  cy_ <- c(1997, 2000, 2003)[cp_]

  for (agr_ in 0:4) {

    # run conventional period analysis
    res_ <- period(subset(colorectum.df, agr==agr_) , 5,
                surv.probs.males, surv.probs.females, cy_, cy_+2)

    obs_ <- rowSums(res_$obs.deaths[,,1])
    py_ <- rowSums(res_$person.years[,,1])
    exp_ <- rowSums(res_$exp.deaths[,,1])

    # append values
    y <- c(y, 1:5)
    cp <- c(cp, rep(cp_, 5))
    agr <- c(agr, rep(agr_, 5))
    obs <- c(obs, obs_)
    dstar <- c(dstar, -(py_-obs_/2)*log((py_-exp_)/py_))
    at_risk <- c(at_risk, log(py_-obs_/2))
  }
}


## couple vectors of model variables in 'model2.df'

model2.df <- data.frame(y=as.factor(y), cp, agr=as.factor(agr),
                obs, dstar, at_risk)


## fit model

model2.fit <- glm(obs ~ y + agr + cp - 1, offset=at_risk,
            family=poisson(modperiod.link(model2.df$dstar)), data=model2.df)

summary(model2.fit)


## load package 'lmtest'

require(lmtest)


## carry out Wald test (H0: cp==0; H1: cp<>0)

waldtest(model2.fit, "cp", test="Chisq")
}


\keyword{survival}



